#include <iostream>
#include <vector>

using namespace std;

class DSU {
public:
    DSU(int count) {
        _dsu_info = vector<int>(count, -1);
        _colors = vector<int>(count, 0);
        _count = count;
    }

    int find(int vertex) {
        return _dsu_info[vertex] < 0 ? vertex : (_dsu_info[vertex] = find(_dsu_info[vertex]));
    }

    bool can_join(int vertex1, int vertex2) {
        return _colors[vertex1] != _colors[vertex2] || _colors[vertex1] == 0 || _colors[vertex2] == 0;
    }

    int join(int vertex1, int vertex2) {
        int set1 = find(vertex1);
        int set2 = find(vertex2);

        if (set1 == set2) {
            return set1;
        }

        int rank1 = -_dsu_info[set1];
        int rank2 = -_dsu_info[set2];

        if (rank1 < rank2) {
            set1 ^= set2;
            set2 ^= set1;
            set1 ^= set2;
        }

        _dsu_info[set1] -= (rank1 == rank2 ? 1 : 0);
        _dsu_info[set2] = set1;

        if (_colors[vertex1] == 0 && _colors[vertex2] == 0) {
            _colors[vertex1] = 1;
            _colors[vertex2] = -1;
        } else if (_colors[vertex1] == 0) {
            _colors[vertex1] = -_colors[vertex2];
        } else if (_colors[vertex2] == 0) {
            _colors[vertex2] = -_colors[vertex1];
        }

        return set1;
    }

    void clear() {
        _dsu_info = vector<int>(_count, -1);
        _colors = vector<int>(_count, 0);
    }
private:
    vector<int> _dsu_info;
    vector<int> _colors;
    int _count;
};

int main() {
    int friends, conflicts_count;
    cin >> friends >> conflicts_count;

    int groups = 1;
    DSU dsu(friends);

    for (int i = 0; i < conflicts_count; i++) {
        int friend1, friend2;
        cin >> friend1 >> friend2;

        friend1 -= 1;
        friend2 -= 1;

        if (dsu.can_join(friend1, friend2)) {
            dsu.join(friend1, friend2);
        } else {
            groups += 1;
            dsu.clear();
            dsu.join(friend1, friend2);
        }
    }

    cout << groups << "\n";

    return 0;
}
